@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// ONELAB - Copyright (C) 2010-2012 C. Geuzaine, F. Henrotte
 //
-// See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// See the README.txt file for license information. Please report all
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -14,7 +14,7 @@
 
 #define ALWAYS_TRUE 1
 
-//onelab::remoteNetworkClient *OLMsg::loader = 0;
+// onelab::remoteNetworkClient *OLMsg::loader = 0;
 
 int OLMsg::_commRank = 0;
 int OLMsg::_commSize = 1;
@@ -29,17 +29,17 @@ std::string OLMsg::_commandLine;
 std::string OLMsg::_launchDate;
 GmshClient *OLMsg::_client = 0;
 onelab::client *OLMsg::_onelabClient = 0;
-bool OLMsg::hasGmsh=false;
+bool OLMsg::hasGmsh = false;
 std::set<std::string, fullNameLessThan> OLMsg::_fullNameDict;
 void (*OLMsg::gui_wait_fct)(double time) = 0;
 
-#if defined(_MSC_VER) && (_MSC_VER >= 1310) //NET 2003
+#if defined(_MSC_VER) && (_MSC_VER >= 1310) // NET 2003
 #define vsnprintf _vsnprintf
 #else
 #if defined(HAVE_NO_VSNPRINTF)
 static int vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
 {
-  if(strlen(fmt) > size - 1){ // just copy the format
+  if(strlen(fmt) > size - 1) { // just copy the format
     strncpy(str, fmt, size - 1);
     str[size - 1] = '\0';
     return size;
@@ -49,8 +49,6 @@ static int vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
 #endif
 #endif
 
-
-
 void OLMsg::Init(int argc, char **argv)
 {
   time_t now;
@@ -58,7 +56,7 @@ void OLMsg::Init(int argc, char **argv)
   _launchDate = ctime(&now);
   _launchDate.resize(_launchDate.size() - 1);
   _commandLine.clear();
-  for(int i = 0; i < argc; i++){
+  for(int i = 0; i < argc; i++) {
     if(i) _commandLine += " ";
     _commandLine += argv[i];
   }
@@ -66,9 +64,7 @@ void OLMsg::Init(int argc, char **argv)
 
 void OLMsg::Exit(int level)
 {
-  if(level){
-    exit(level);
-  }
+  if(level) { exit(level); }
   exit(_errorCount);
 }
 
@@ -82,22 +78,21 @@ void OLMsg::Fatal(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback) (*_callback)("Fatal", str);
-  //if(_client) _client->Error(str);
+  // if(_callback) (*_callback)("Fatal", str);
+  // if(_client) _client->Error(str);
 
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendError(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendError(str);
 
-  if(ALWAYS_TRUE){
+  if(ALWAYS_TRUE) {
     if(_commSize > 1)
       fprintf(stderr, "Fatal   : [On processor %d] %s\n", _commRank, str);
     else
       fprintf(stderr, "Fatal   : %s\n", str);
-   fflush(stderr);
+    fflush(stderr);
   }
-  //FinalizeClient();
+  // FinalizeClient();
   FinalizeOnelab();
-  //delete loader;
+  // delete loader;
   Exit(1);
 }
 
@@ -113,12 +108,11 @@ void OLMsg::Error(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback) (*_callback)("Error", str);
-  //if(_client) _client->Error(str);
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendError(str);
+  // if(_callback) (*_callback)("Error", str);
+  // if(_client) _client->Error(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendError(str);
 
-  if(ALWAYS_TRUE){
+  if(ALWAYS_TRUE) {
     if(_commSize > 1)
       fprintf(stderr, "Error   : [On processor %d] %s\n", _commRank, str);
     else
@@ -139,12 +133,11 @@ void OLMsg::Warning(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback) (*_callback)("Warning", str);
-  //if(_client) _client->Warning(str);
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendWarning(str);
+  // if(_callback) (*_callback)("Warning", str);
+  // if(_client) _client->Warning(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendWarning(str);
 
-  if(ALWAYS_TRUE){
+  if(ALWAYS_TRUE) {
     fprintf(stderr, "Warning : %s\n", str);
     fflush(stderr);
   }
@@ -160,12 +153,11 @@ void OLMsg::Info(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback) (*_callback)("Info", str);
-  //if(_client) _client->Info(str);
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendInfo(str);
+  // if(_callback) (*_callback)("Info", str);
+  // if(_client) _client->Info(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendInfo(str);
 
-  if(ALWAYS_TRUE){
+  if(ALWAYS_TRUE) {
     fprintf(stdout, "Onelab  : %s\n", str);
     fflush(stdout);
   }
@@ -194,12 +186,11 @@ void OLMsg::Direct(int level, const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback) (*_callback)("Direct", str);
-  //if(_client) _client->Info(str);
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendInfo(str);
+  // if(_callback) (*_callback)("Direct", str);
+  // if(_client) _client->Info(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendInfo(str);
 
-  if(ALWAYS_TRUE){
+  if(ALWAYS_TRUE) {
     fprintf(stdout, "%s\n", str);
     fflush(stdout);
   }
@@ -216,13 +207,12 @@ void OLMsg::StatusBar(int num, bool log, const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  //if(_callback && log) (*_callback)("Info", str);
-  //if(_client && log) _client->Info(str);
+  // if(_callback && log) (*_callback)("Info", str);
+  // if(_client && log) _client->Info(str);
 
-  if(_onelabClient && OLMsg::hasGmsh)
-    _onelabClient->sendInfo(str);
+  if(_onelabClient && OLMsg::hasGmsh) _onelabClient->sendInfo(str);
 
-  if(log && ALWAYS_TRUE){
+  if(log && ALWAYS_TRUE) {
     fprintf(stdout, "Info    : %s\n", str);
     fflush(stdout);
   }
@@ -235,30 +225,25 @@ void OLMsg::InitializeOnelab(const std::string &name)
   OLMsg::hasGmsh = OLMsg::GetOnelabNumber("IsMetamodel");
 }
 
-void OLMsg::SetGuiWaitFunction(void (*fct)(double time)){
-  gui_wait_fct = fct;
-}
-void (*OLMsg::GetGuiWaitFunction())(double){
-  return gui_wait_fct;
-}
+void OLMsg::SetGuiWaitFunction(void (*fct)(double time)) { gui_wait_fct = fct; }
+void (*OLMsg::GetGuiWaitFunction())(double) { return gui_wait_fct; }
 
 double OLMsg::GetOnelabNumber(std::string name)
 {
-  if(_onelabClient){
+  if(_onelabClient) {
     std::vector<onelab::number> ps;
     _onelabClient->get(ps, name);
-    if(ps.size())
-      return ps[0].getValue();
+    if(ps.size()) return ps[0].getValue();
   }
   return 0;
 }
 
 void OLMsg::GetOnelabNumber(std::string name, double *val)
 {
-  if(_onelabClient){
+  if(_onelabClient) {
     std::vector<onelab::number> ps;
     _onelabClient->get(ps, name);
-    if(ps.size()){
+    if(ps.size()) {
       *val = ps[0].getValue();
       return;
     }
@@ -268,10 +253,10 @@ void OLMsg::GetOnelabNumber(std::string name, double *val)
 
 void OLMsg::SetOnelabNumber(std::string name, double val, bool visible)
 {
-  if(_onelabClient){
+  if(_onelabClient) {
     std::vector<onelab::number> numbers;
     _onelabClient->get(numbers, name);
-    if(numbers.empty()){
+    if(numbers.empty()) {
       numbers.resize(1);
       numbers[0].setName(name);
     }
@@ -283,22 +268,23 @@ void OLMsg::SetOnelabNumber(std::string name, double val, bool visible)
 
 std::string OLMsg::GetOnelabString(std::string name)
 {
-  std::string str="";
-  if(_onelabClient){
+  std::string str = "";
+  if(_onelabClient) {
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
-    if(ps.size() && ps[0].getValue().size())
-      str = ps[0].getValue();
+    if(ps.size() && ps[0].getValue().size()) str = ps[0].getValue();
   }
   return str;
 }
 
-bool OLMsg::GetOnelabChoices(std::string name, std::vector<std::string> &choices){
-  if(_onelabClient){
+bool OLMsg::GetOnelabChoices(std::string name,
+                             std::vector<std::string> &choices)
+{
+  if(_onelabClient) {
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
-    if(ps.size() && ps[0].getValue().size()){
-      choices=ps[0].getChoices();
+    if(ps.size() && ps[0].getValue().size()) {
+      choices = ps[0].getChoices();
       return true;
     }
   }
@@ -307,130 +293,139 @@ bool OLMsg::GetOnelabChoices(std::string name, std::vector<std::string> &choices
 
 void OLMsg::SetOnelabString(std::string name, std::string val, bool visible)
 {
-  if(_onelabClient){
+  if(_onelabClient) {
     std::vector<onelab::string> strings;
     _onelabClient->get(strings, name);
-    if(strings.empty()){
+    if(strings.empty()) {
       strings.resize(1);
       strings[0].setName(name);
     }
     strings[0].setValue(val);
     strings[0].setVisible(visible);
-    strings[0].setAttribute("NotInDb","True");
+    strings[0].setAttribute("NotInDb", "True");
     _onelabClient->set(strings[0]);
   }
 }
 
 void OLMsg::SetVisible(std::string name, bool visible)
 {
-  if(_onelabClient){
+  if(_onelabClient) {
     std::vector<onelab::string> strings;
     _onelabClient->get(strings, name);
-    if(strings.size()){
+    if(strings.size()) {
       strings[0].setVisible(visible);
       _onelabClient->set(strings[0]);
     }
   }
 }
 
-void OLMsg::SetOnelabAttributeString(std::string name,
-				   std::string attrib,std::string val){
-  if(_onelabClient){
+void OLMsg::SetOnelabAttributeString(std::string name, std::string attrib,
+                                     std::string val)
+{
+  if(_onelabClient) {
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
-    if(ps.size()){
-      ps[0].setAttribute(attrib,val);
-    }
+    if(ps.size()) { ps[0].setAttribute(attrib, val); }
   }
 }
-std::string OLMsg::GetOnelabAttributeString(std::string name,std::string attrib){
-  std::string str="";
-  if(_onelabClient){
+std::string OLMsg::GetOnelabAttributeString(std::string name,
+                                            std::string attrib)
+{
+  std::string str = "";
+  if(_onelabClient) {
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
-    if(ps.size())
-      str = ps[0].getAttribute(attrib);
+    if(ps.size()) str = ps[0].getAttribute(attrib);
   }
   return str;
 }
-std::string OLMsg::GetOnelabAttributeNumber(std::string name,std::string attrib){
-  std::string str="";
-  if(_onelabClient){
+std::string OLMsg::GetOnelabAttributeNumber(std::string name,
+                                            std::string attrib)
+{
+  std::string str = "";
+  if(_onelabClient) {
     std::vector<onelab::number> ps;
     _onelabClient->get(ps, name);
-    if(ps.size())
-      str = ps[0].getAttribute(attrib);
+    if(ps.size()) str = ps[0].getAttribute(attrib);
   }
   return str;
 }
 
-int fullNameLessThan::compareFullNames(const std::string a, const std::string b) const{
+int fullNameLessThan::compareFullNames(const std::string a,
+                                       const std::string b) const
+{
   std::string::const_iterator ita, itb;
 
   // Compares the strings a and b
   // One-digit numbers at the beginning of the string
   // or directly following the separator '/' are ignored
 
-  ita = a.begin(); itb = b.begin();
+  ita = a.begin();
+  itb = b.begin();
 
   // ignore a possible initial one-digit number
-  if( (ita < a.end()) && (*ita >= '0') && (*ita <= '9')) ita++;
-  if( (itb < b.end()) && (*itb >= '0') && (*itb <= '9')) itb++;
+  if((ita < a.end()) && (*ita >= '0') && (*ita <= '9')) ita++;
+  if((itb < b.end()) && (*itb >= '0') && (*itb <= '9')) itb++;
 
-  while( (ita < a.end()) && (itb < b.end())) {
-    if(*ita == *itb){
-      if(*ita == '/'){ // hence *itb == '/'
-	ita++; if( (ita < a.end()) && (*ita >= '0') && (*ita <= '9')) ita++;
-	itb++; if( (itb < b.end()) && (*itb >= '0') && (*itb <= '9')) itb++;
+  while((ita < a.end()) && (itb < b.end())) {
+    if(*ita == *itb) {
+      if(*ita == '/') { // hence *itb == '/'
+        ita++;
+        if((ita < a.end()) && (*ita >= '0') && (*ita <= '9')) ita++;
+        itb++;
+        if((itb < b.end()) && (*itb >= '0') && (*itb <= '9')) itb++;
       }
-      else{
-	ita++;
-	itb++;
+      else {
+        ita++;
+        itb++;
       }
     }
     else { // mismatched character found
-      return *ita < *itb ;
+      return *ita < *itb;
     }
   }
   // either string is at end()
   return !(itb == b.end());
 }
 
-void OLMsg::recordFullName(const std::string &name){
+void OLMsg::recordFullName(const std::string &name)
+{
   OLMsg::_fullNameDict.insert(name);
 }
 
-std::string OLMsg::obtainFullName(const std::string &name){
+std::string OLMsg::obtainFullName(const std::string &name)
+{
   std::set<std::string, fullNameLessThan>::iterator it;
 
   // fullNameLessThan* comp=new fullNameLessThan;
   // std::cout << "Dict=" << OLMsg::_fullNameDict.size() << std::endl;
   // std::cout << "Looking for " << name << std::endl;
-  // for ( it=OLMsg::_fullNameDict.begin() ; it != OLMsg::_fullNameDict.end(); it++ )
-  //   std::cout << *it << " <" << comp->operator()(*it,name) << ">" << std::endl;
+  // for ( it=OLMsg::_fullNameDict.begin() ; it != OLMsg::_fullNameDict.end();
+  // it++ )
+  //   std::cout << *it << " <" << comp->operator()(*it,name) << ">" <<
+  //   std::endl;
   // std::cout << std::endl;
 
   it = OLMsg::_fullNameDict.find(name);
-  if(it == OLMsg::_fullNameDict.end()){
-    return name;
-  }
-  else{
+  if(it == OLMsg::_fullNameDict.end()) { return name; }
+  else {
     return *it;
   }
 }
 
-void OLMsg::MergeFile(const std::string &name){
-  //Sends files (geo, pos, msh) to Gmsh
-  //The parameter Gmsh/MergedGeo ensures that the geometry is sent once.
-  if(_onelabClient){
-    if(name.find(".geo") != std::string::npos){
-      if(GetOnelabString("Gmsh/MergedGeo").empty()){
-	SetOnelabString("Gmsh/MergedGeo",name,false);
-	Info("Merge a geometry <%s> to Gmsh", name.c_str());
-	_onelabClient->sendMergeFileRequest(name);
+void OLMsg::MergeFile(const std::string &name)
+{
+  // Sends files (geo, pos, msh) to Gmsh
+  // The parameter Gmsh/MergedGeo ensures that the geometry is sent once.
+  if(_onelabClient) {
+    if(name.find(".geo") != std::string::npos) {
+      if(GetOnelabString("Gmsh/MergedGeo").empty()) {
+        SetOnelabString("Gmsh/MergedGeo", name, false);
+        Info("Merge a geometry <%s> to Gmsh", name.c_str());
+        _onelabClient->sendMergeFileRequest(name);
       }
     }
-    else{
+    else {
       Info("Merge <%s> to Gmsh", name.c_str());
       _onelabClient->sendMergeFileRequest(name);
     }
@@ -439,9 +434,9 @@ void OLMsg::MergeFile(const std::string &name){
     OLMsg::Info("Not connected to Gmsh");
 }
 
-
-void OLMsg::FinalizeOnelab(){
-  if(_onelabClient){
+void OLMsg::FinalizeOnelab()
+{
+  if(_onelabClient) {
     delete _onelabClient;
     _onelabClient = 0;
     _client = 0;
