@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -20,19 +20,28 @@ static void drawEntityLabel(drawContext *ctx, GEntity *e, double x, double y,
   double yy = y + offset / ctx->s[1];
   double zz = z + offset / ctx->s[2];
 
-  if(CTX::instance()->geom.labelType == 0) {
-    std::vector<std::string> info =
-      SplitString(e->getInfoString(false, true), '\n');
-    for(int line = 0; line < (int)info.size(); line++)
-      ctx->drawString(info[line].c_str(), xx, yy, zz, line);
-    return;
-  }
-
   char str[1024];
-  if(CTX::instance()->geom.labelType == 1) {
-    sprintf(str, "%d", e->tag());
-  }
-  else if(CTX::instance()->geom.labelType == 2) {
+  switch(CTX::instance()->geom.labelType) {
+  case 5:
+    {
+      sprintf(str, "(%g,%g,%g)", x, y, z);
+    }
+    break;
+  case 4:
+    {
+      strcpy(str, "");
+      std::string name = "";
+      for(std::size_t i = 0; i < e->physicals.size(); i++) {
+        if(name.size()) strcat(str, ", ");
+        name = e->model()->getPhysicalName(e->dim(), std::abs(e->physicals[i]));
+        if(name.size()) strcat(str, name.c_str());
+      }
+    }
+    break;
+  case 3:
+    strcpy(str, e->model()->getElementaryName(e->dim(), e->tag()).c_str());
+    break;
+  case 2:
     strcpy(str, "");
     for(std::size_t i = 0; i < e->physicals.size(); i++) {
       char tmp[32];
@@ -40,18 +49,19 @@ static void drawEntityLabel(drawContext *ctx, GEntity *e, double x, double y,
       sprintf(tmp, "%d", e->physicals[i]);
       strcat(str, tmp);
     }
-  }
-  else if(CTX::instance()->geom.labelType == 3) {
-    strcpy(str, e->model()->getElementaryName(e->dim(), e->tag()).c_str());
-  }
-  else {
-    strcpy(str, "");
-    std::string name = "";
-    for(std::size_t i = 0; i < e->physicals.size(); i++) {
-      if(name.size()) strcat(str, ", ");
-      name = e->model()->getPhysicalName(e->dim(), std::abs(e->physicals[i]));
-      if(name.size()) strcat(str, name.c_str());
+    break;
+  case 1:
+    sprintf(str, "%d", e->tag());
+    break;
+  case 0:
+  default:
+    {
+      std::vector<std::string> info =
+        SplitString(e->getInfoString(false, true), '\n');
+      for(int line = 0; line < (int)info.size(); line++)
+        ctx->drawString(info[line].c_str(), xx, yy, zz, line);
     }
+    return;
   }
 
   ctx->drawString(str, xx, yy, zz);
@@ -97,10 +107,9 @@ public:
     }
 
     if(CTX::instance()->geom.highlightOrphans) {
-      std::vector<GEdge *> const &edges = v->edges();
-      if(edges.size() == 0)
+      if(v->isOrphan())
         glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[0]);
-      else if(edges.size() == 1)
+      else if(v->numEdges() == 1)
         glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[1]);
     }
 
@@ -174,10 +183,9 @@ public:
     }
 
     if(CTX::instance()->geom.highlightOrphans) {
-      std::vector<GFace *> faces = e->faces();
-      if(faces.empty())
+      if(e->isOrphan())
         glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[0]);
-      else if(faces.size() == 1)
+      else if(e->numFaces() == 1)
         glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[1]);
     }
 
@@ -329,6 +337,13 @@ public:
       glColor4ubv((GLubyte *)&col);
     }
 
+    if(CTX::instance()->geom.highlightOrphans) {
+      if(f->isOrphan())
+        glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[0]);
+      else if(f->numRegions() == 1)
+        glColor4ubv((GLubyte *)&CTX::instance()->color.geom.highlight[1]);
+    }
+
     if(CTX::instance()->geom.lightTwoSide)
       glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
     else
@@ -433,29 +448,60 @@ public:
     else
       glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
 
-    if(r->getSelection()){
+    if(r->getSelection()) {
+      glLineWidth((float)CTX::instance()->geom.selectedCurveWidth);
+      gl2psLineWidth((float)(CTX::instance()->geom.selectedCurveWidth *
+                             CTX::instance()->print.epsLineWidthFactor));
       glColor4ubv((GLubyte *)&CTX::instance()->color.geom.selection);
     }
-    else{
+    else {
+      glLineWidth((float)CTX::instance()->geom.curveWidth);
+      gl2psLineWidth((float)(CTX::instance()->geom.curveWidth *
+                             CTX::instance()->print.epsLineWidthFactor));
       unsigned int col = r->useColor() ? r->getColor() :
         CTX::instance()->color.geom.volume;
       glColor4ubv((GLubyte *)&col);
     }
 
     const double size = 8.;
-    double x = 0., y = 0., z = 0.;
+    double x = 0., y = 0., z = 0., d = 0.;
 
     if(CTX::instance()->geom.volumes || CTX::instance()->geom.volumeLabels ||
        r->getSelection() > 1) {
-      SPoint3 p = r->bounds(true).center(); // fast approx if mesh-based
+      SBoundingBox3d bb = r->bounds(true); // fast approx if mesh-based
+      SPoint3 p = bb.center();
       x = p.x();
       y = p.y();
       z = p.z();
+      d = bb.diag() / 50.;
       _ctx->transform(x, y, z);
     }
 
-    if(CTX::instance()->geom.volumes || r->getSelection() > 1)
-      _ctx->drawSphere(size, x, y, z, CTX::instance()->geom.light);
+    if(CTX::instance()->geom.volumes || r->getSelection() > 1) {
+      if(CTX::instance()->geom.volumeType == 0) {
+        _ctx->drawSphere(size, x, y, z, CTX::instance()->geom.light);
+      }
+      else {
+        glBegin(GL_LINE_LOOP);
+        glVertex3d(x + d, y, z);
+        glVertex3d(x, y + d, z);
+        glVertex3d(x - d, y, z);
+        glVertex3d(x, y - d, z);
+        glEnd();
+        glBegin(GL_LINE_LOOP);
+        glVertex3d(x + d, y, z);
+        glVertex3d(x, y, z + d);
+        glVertex3d(x - d, y, z);
+        glVertex3d(x, y, z - d);
+        glEnd();
+        glBegin(GL_LINE_LOOP);
+        glVertex3d(x, y + d, z);
+        glVertex3d(x, y, z + d);
+        glVertex3d(x, y - d, z);
+        glVertex3d(x, y, z - d);
+        glEnd();
+      }
+    }
 
     if(CTX::instance()->geom.volumeLabels || r->getSelection() > 1) {
       double offset =
