@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -993,7 +993,6 @@ void delaunayTrgl(const std::size_t numThreads, const std::size_t NPTS_AT_ONCE,
 #endif
 
     double totSearch = 0;
-    double totCavity = 0;
     std::vector<std::size_t> _negatives;
     std::vector<cavityContainer> cavity(NPTS_AT_ONCE);
     std::vector<connContainer> bnd(NPTS_AT_ONCE);
@@ -1076,7 +1075,6 @@ void delaunayTrgl(const std::size_t numThreads, const std::size_t NPTS_AT_ONCE,
           faceToTet.clear();
           const std::size_t cSize = cavityK.size();
           const std::size_t bSize = bndK.size();
-          totCavity += cSize;
           Choice[K] = cavityK[0];
           for(std::size_t i = 0; i < bSize; i++) {
             // reuse memory slots of invalid elements
@@ -1113,7 +1111,7 @@ void delaunayTrgl(const std::size_t numThreads, const std::size_t NPTS_AT_ONCE,
       }
     }
 #if defined(VERBOSE)
-#pragma omp critical
+#pragma omp critical(delaunayTrgl)
     {
       totCavityGlob += totCavity;
       totSearchGlob += totSearch;
@@ -1264,7 +1262,7 @@ void delaunayTriangulation(const int numThreads, const int nptsatonce,
     _temp[v->getNum()] = mv;
   }
 
-  robustPredicates::exactinit(1, maxx, maxy, maxz);
+  robustPredicates::exactinit(maxx, maxy, maxz);
 
   Vert *box[8];
   delaunayTriangulation(numThreads, nptsatonce, _vertices, box, allocator);
