@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2025 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -57,6 +57,15 @@ RecombineTriangle::RecombineTriangle(const MEdge &me, MElement *_t1,
   else if(t2->getVertex(2) != n1 && t2->getVertex(2) != n2)
     n4 = t2->getVertex(2);
 
+  if(!n3) {
+    Msg::Warning("Invalid quadrangle in recombination");
+    n3 = n1;
+  }
+  if(!n4) {
+    Msg::Warning("Invalid quadrangle in recombination");
+    n4 = n2;
+  }
+
   MQuadrangle q(n1, n3, n2, n4);
   angle = q.etaShapeMeasure();
 
@@ -612,10 +621,10 @@ static bool _tryToCollapseThatVertex2(GFace *gf, std::vector<MElement *> &e1,
   double worst_quality_old = 1.0;
   double worst_quality_new = 1.0;
 
-  int count = 0;
+  //int count = 0;
   for(std::size_t j = 0; j < e.size(); ++j) {
     if(e[j] != q) {
-      count++;
+      //count++;
       worst_quality_old = std::min(worst_quality_old, e[j]->etaShapeMeasure());
       v1->x() = pp.x();
       v1->y() = pp.y();
@@ -677,10 +686,10 @@ static bool _tryToCollapseThatVertex(GFace *gf, std::vector<MElement *> &e1,
   GPoint pp = gf->point(0.5 * (uu1 + uu2), 0.5 * (vv1 + vv2));
   double worst_quality_old = 1.0;
   double worst_quality_new = 1.0;
-  int count = 0;
+  //int count = 0;
   for(std::size_t j = 0; j < e.size(); ++j) {
     if(e[j] != q) {
-      count++;
+      //count++;
       worst_quality_old = std::min(worst_quality_old, e[j]->etaShapeMeasure());
       v1->x() = pp.x();
       v1->y() = pp.y();
@@ -762,6 +771,17 @@ static bool _isItAGoodIdeaToMoveThatVertex(GFace *gf,
   return true;
 }
 
+void getAllBoundaryLayerVertices(GFace *gf, std::set<MVertex *> &vs)
+{
+  vs.clear();
+  BoundaryLayerColumns *_columns = gf->getColumns();
+  if(!_columns) return;
+  for(auto it = _columns->_data.begin(); it != _columns->_data.end(); it++) {
+    BoundaryLayerData &data = it->second;
+    for(size_t i = 0; i < data._column.size(); i++) vs.insert(data._column[i]);
+  }
+}
+
 static bool has_none_of(std::set<MVertex *> const &touched, MVertex *const v1,
                         MVertex *const v2, MVertex *const v3, MVertex *const v4)
 {
@@ -801,6 +821,10 @@ static int _removeDiamonds(GFace *const gf)
     touched.insert(gf->triangles[i]->getVertex(2));
   }
 
+  std::set<MVertex *> vs;
+  getAllBoundaryLayerVertices(gf, vs);
+  touched.insert(vs.begin(), vs.end());
+
   for(std::size_t i = 0; i < gf->quadrangles.size(); i++) {
     MQuadrangle *const q = gf->quadrangles[i];
 
@@ -955,17 +979,6 @@ static void _relocate(GFace *gf, MVertex *ver,
   }
 }
 
-void getAllBoundaryLayerVertices(GFace *gf, std::set<MVertex *> &vs)
-{
-  vs.clear();
-  BoundaryLayerColumns *_columns = gf->getColumns();
-  if(!_columns) return;
-  for(auto it = _columns->_data.begin(); it != _columns->_data.end(); it++) {
-    BoundaryLayerData &data = it->second;
-    for(size_t i = 0; i < data._column.size(); i++) vs.insert(data._column[i]);
-  }
-}
-
 void laplaceSmoothing(GFace *gf, int niter, bool infinity_norm)
 {
   if((gf->triangles.size() > 0 && gf->triangles[0]->getPolynomialOrder() > 1) ||
@@ -1002,63 +1015,27 @@ static void _recombineIntoQuads(GFace *gf, bool blossom, bool cubicGraph = 1)
   if(gf->triangles.empty()) return;
   if(gf->compound.size()) return;
 
-  std::vector<MVertex *> emb_edgeverts;
-  {
+  std::set<MEdge,MEdgeLessThan> bndedges;
+  std::set<MEdge,MEdgeLessThan> embedges;
+  {  
     std::vector<GEdge *> emb_edges = gf->getEmbeddedEdges();
-    auto ite = emb_edges.begin();
-    while(ite != emb_edges.end()) {
-      if(!(*ite)->isMeshDegenerated()) {
-        emb_edgeverts.insert(emb_edgeverts.end(), (*ite)->mesh_vertices.begin(),
-                             (*ite)->mesh_vertices.end());
-        if((*ite)->getBeginVertex())
-          emb_edgeverts.insert(emb_edgeverts.end(),
-                               (*ite)->getBeginVertex()->mesh_vertices.begin(),
-                               (*ite)->getBeginVertex()->mesh_vertices.end());
-        if((*ite)->getEndVertex())
-          emb_edgeverts.insert(emb_edgeverts.end(),
-                               (*ite)->getEndVertex()->mesh_vertices.begin(),
-                               (*ite)->getEndVertex()->mesh_vertices.end());
-      }
-      ++ite;
-    }
+    for (auto ee : emb_edges)
+      for (auto l : ee->lines)
+    	embedges.insert (MEdge(l->getVertex(0),l->getVertex(1)));
+    
+    std::vector<GEdge *> edges = gf->edges();
+    for (auto ee : edges)
+      for (auto l : ee->lines)
+	bndedges.insert (MEdge(l->getVertex(0),l->getVertex(1)));
   }
 
-  {
-    std::vector<GEdge *> const &_edges = gf->edges();
-    auto ite = _edges.begin();
-    while(ite != _edges.end()) {
-      if(!(*ite)->isMeshDegenerated()) {
-        if((*ite)->isSeam(gf)) {
-          emb_edgeverts.insert(emb_edgeverts.end(),
-                               (*ite)->mesh_vertices.begin(),
-                               (*ite)->mesh_vertices.end());
-          if((*ite)->getBeginVertex())
-            emb_edgeverts.insert(
-              emb_edgeverts.end(),
-              (*ite)->getBeginVertex()->mesh_vertices.begin(),
-              (*ite)->getBeginVertex()->mesh_vertices.end());
-          if((*ite)->getEndVertex())
-            emb_edgeverts.insert(emb_edgeverts.end(),
-                                 (*ite)->getEndVertex()->mesh_vertices.begin(),
-                                 (*ite)->getEndVertex()->mesh_vertices.end());
-        }
-      }
-      ++ite;
-    }
-  }
-
-  // sort and erase the duplicates
-  std::sort(emb_edgeverts.begin(), emb_edgeverts.end());
-  emb_edgeverts.erase(std::unique(emb_edgeverts.begin(), emb_edgeverts.end()),
-                      emb_edgeverts.end());
-
   e2t_cont adj;
   buildEdgeToElement(gf->triangles, adj);
 
   FieldManager *fields = gf->model()->getFields();
   Field *cross_field = NULL;
   SVector3 t1;
-  // double L; /* unused ? */
+
   if(fields->getBackgroundField() > 0) {
     cross_field = fields->get(fields->getBackgroundField());
     if(cross_field->numComponents() != 3) { // we hae a true scaled cross fiel
@@ -1071,16 +1048,15 @@ static void _recombineIntoQuads(GFace *gf, bool blossom, bool cubicGraph = 1)
   std::map<MVertex *, std::pair<MElement *, MElement *> > makeGraphPeriodic;
 
   for(auto it = adj.begin(); it != adj.end(); ++it) {
-    if(it->second.second && it->second.first->getNumVertices() == 3 &&
-       it->second.second->getNumVertices() == 3 &&
-       (!std::binary_search(emb_edgeverts.begin(), emb_edgeverts.end(),
-                            it->first.getVertex(0)) ||
-        !std::binary_search(emb_edgeverts.begin(), emb_edgeverts.end(),
-                            it->first.getVertex(1)))) {
+    MEdge ed (it->first.getVertex(0),it->first.getVertex(1));
+    if(bndedges.find(ed) == bndedges.end() && 
+       it->second.second && it->second.first->getNumVertices() == 3 &&
+       it->second.second->getNumVertices() == 3) {
       pairs.push_back(RecombineTriangle(it->first, it->second.first,
-                                        it->second.second, cross_field));
+					it->second.second, cross_field));
     }
-    else if(!it->second.second && it->second.first->getNumVertices() == 3) {
+    else if(embedges.find(ed) == embedges.end() &&
+	    !it->second.second && it->second.first->getNumVertices() == 3) {
       for(int i = 0; i < 2; i++) {
         MVertex *const v = it->first.getVertex(i);
         auto itv = makeGraphPeriodic.find(v);
@@ -1158,10 +1134,11 @@ static void _recombineIntoQuads(GFace *gf, bool blossom, bool cubicGraph = 1)
       sprintf(MATCHFILE, ".face.match");
       if(perfect_match(ncount, nullptr, ecount, &elist, &elen, nullptr,
                        MATCHFILE, 0, 0, 0, 0, &matzeit)) {
-        Msg::Error(
+        Msg::Warning(
           "Perfect Match failed in quadrangulation, try something else");
         free(elist);
         pairs.clear();
+	_recombineIntoQuads(gf, false, cubicGraph);
       }
       else {
         // TEST
@@ -1198,10 +1175,26 @@ static void _recombineIntoQuads(GFace *gf, bool blossom, bool cubicGraph = 1)
                 break;
               }
             }
-            MQuadrangle *q = new MQuadrangle(
-              t1->getVertex(start), t1->getVertex((start + 1) % 3), other,
-              t1->getVertex((start + 2) % 3));
-            gf->quadrangles.push_back(q);
+	    MVertex *vs[4] = {t1->getVertex(start), t1->getVertex((start + 1) % 3), other,
+			      t1->getVertex((start + 2) % 3)};
+	    MEdge e1 (vs[0],vs[2]);
+	    MEdge e2 (vs[1],vs[3]);
+	    if (embedges.find(e1) != embedges.end()){
+	      MTriangle *t1 = new MTriangle(vs[0],vs[1],vs[2]);
+	      MTriangle *t2 = new MTriangle(vs[2],vs[3],vs[0]);
+	      gf->triangles.push_back(t1);
+	      gf->triangles.push_back(t2);
+	    }
+	    else if (embedges.find(e2) != embedges.end()){
+	      MTriangle *t1 = new MTriangle(vs[1],vs[2],vs[3]);
+	      MTriangle *t2 = new MTriangle(vs[3],vs[0],vs[1]);
+	      gf->triangles.push_back(t1);
+	      gf->triangles.push_back(t2);
+	    }
+	    else {
+	      MQuadrangle *q = new MQuadrangle(vs[0],vs[1],vs[2],vs[3]);
+	      gf->quadrangles.push_back(q);
+	    }
           }
         }
         free(elist);
@@ -1318,29 +1311,34 @@ void recombineIntoQuads(GFace *gf, bool blossom, int topologicalOptiPasses,
     if(debug) gf->model()->writeMSH("recombine_2smoothed.msh");
   }
 
-  if(topologicalOptiPasses > 0) {
-    if(!_isModelOkForTopologicalOpti(gf->model())) {
-      Msg::Info(
-        "Skipping topological optimization - mesh topology is not complete");
-    }
-    else {
-      int iter = 0, nbTwoQuadNodes = 1, nbDiamonds = 1;
-      while(nbTwoQuadNodes || nbDiamonds) {
-        Msg::Debug("Topological optimization of quad mesh: pass %d", iter);
-        nbTwoQuadNodes = removeTwoQuadsNodes(gf);
-        // removeDiamonds uses the parametrization or searches for closest point
-        nbDiamonds = haveParam ? removeDiamonds(gf) : 0;
-        if(haveParam && nodeRepositioning)
-          RelocateVertices(gf, CTX::instance()->mesh.nbSmoothing);
-        iter++;
-        if(iter > topologicalOptiPasses) break;
+// FIXME: not thread-safe
+#pragma omp critical(recombineIntoQuads)
+  {
+    if(topologicalOptiPasses > 0) {
+      if(!_isModelOkForTopologicalOpti(gf->model())) {
+        Msg::Info
+          ("Skipping topological optimization - mesh topology is not complete");
+      }
+      else {
+        int iter = 0, nbTwoQuadNodes = 1, nbDiamonds = 1;
+        while(nbTwoQuadNodes || nbDiamonds) {
+          Msg::Debug("Topological optimization of quad mesh: pass %d", iter);
+          nbTwoQuadNodes = removeTwoQuadsNodes(gf);
+          // removeDiamonds uses the parametrization or searches for closest point
+          nbDiamonds = haveParam ? removeDiamonds(gf) : 0;
+          if(haveParam && nodeRepositioning)
+            RelocateVertices(gf, CTX::instance()->mesh.nbSmoothing);
+          iter++;
+          if(iter > topologicalOptiPasses) break;
+        }
+        if(debug) gf->model()->writeMSH("recombine_3topo.msh");
       }
-      if(debug) gf->model()->writeMSH("recombine_3topo.msh");
     }
   }
 
   // re-split bad quads into triangles
-  quadsToTriangles(gf, minqual);
+  if(minqual > 0)
+    quadsToTriangles(gf, minqual);
 
   if(debug) gf->model()->writeMSH("recombine_4quality.msh");
 
@@ -1449,8 +1447,9 @@ void splitElementsInBoundaryLayerIfNeeded(GFace *gf)
     int n = fields->getNumBoundaryLayerFields();
     for(int i = 0; i < n; ++i) {
       Field *bl_field = fields->get(fields->getBoundaryLayerField(i));
-      if(bl_field == nullptr) continue;
+      if(!bl_field) continue;
       BoundaryLayerField *blf = dynamic_cast<BoundaryLayerField *>(bl_field);
+      if(!blf) continue;
       if(blf->iRecombine)
         ++numNoSplit;
       else
